unit Core;

interface

uses
  Classes, SysUtils, XMLConf, FileUtil, ActnList, Controls, Dialogs, Forms,
  Game, ApplicationInfo, PersistentForm, ScaleDPI, Translator, DOM,
  RegistryEx, LastOpenedList, Theme, Registry, Menus, FormCharts, FormMain,
  FormClient, Player, GameServer, GameClient, ServerList, GameSystem, Graphics,
  &Unit;

type

  { TCore }

  TCore = class(TDataModule)
    AAbout: TAction;
    AGameSystems: TAction;
    ATests: TAction;
    AFullscreen: TAction;
    AToggleFogOfWar: TAction;
    AShowKeyShortcuts: TAction;
    AGameSave: TAction;
    APlayersStats: TAction;
    ANewSpectatorClient: TAction;
    AShowUnitMoves: TAction;
    AShowCharts: TAction;
    AHelp: TAction;
    AGameSaveAs: TAction;
    AGameLoad: TAction;
    ApplicationInfo: TApplicationInfo;
    ASettings: TAction;
    ActionListMain: TActionList;
    AExit: TAction;
    AGameEnd: TAction;
    AGameNew: TAction;
    AGameRestart: TAction;
    Translator1: TTranslator;
    ImageListLarge: TImageList;
    ImageListSmall: TImageList;
    LastOpenedList1: TLastOpenedList;
    OpenDialog1: TOpenDialog;
    PersistentForm1: TPersistentForm;
    SaveDialog1: TSaveDialog;
    ScaleDPI1: TScaleDPI;
    ThemeManager1: TThemeManager;
    XMLConfig1: TXMLConfig;
    procedure AAboutExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AFullscreenExecute(Sender: TObject);
    procedure AGameEndExecute(Sender: TObject);
    procedure AGameLoadExecute(Sender: TObject);
    procedure AGameNewExecute(Sender: TObject);
    procedure AGameRestartExecute(Sender: TObject);
    procedure AGameSaveAsExecute(Sender: TObject);
    procedure AGameSaveExecute(Sender: TObject);
    procedure AGameSystemsExecute(Sender: TObject);
    procedure AHelpExecute(Sender: TObject);
    procedure ANewSpectatorClientExecute(Sender: TObject);
    procedure APlayersStatsExecute(Sender: TObject);
    procedure ASettingsExecute(Sender: TObject);
    procedure AShowChartsExecute(Sender: TObject);
    procedure AShowKeyShortcutsExecute(Sender: TObject);
    procedure AShowUnitMovesExecute(Sender: TObject);
    procedure ATestsExecute(Sender: TObject);
    procedure AToggleFogOfWarExecute(Sender: TObject);
    procedure Translator1Translate(Sender: TObject);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
    procedure LastOpenedList1Change(Sender: TObject);
  private
    FInitialized: Boolean;
    StoredDimension: TControlDimension;
    RegistryContext: TRegistryContext;
    GameLoaded: Boolean;
    FormNewTabIndex: Integer;
    GameFilesDir: string;
    procedure LoadRecentExecute(Sender: TObject);
    procedure StartNewGame;
    procedure LoadGame(FileName: string);
    procedure DoPlayerChange(Sender: TObject);
    procedure DoMoveUpdated(UnitMove: TUnitMove);
    procedure DoOnWin(Player: TPlayer);
    procedure Delay(Time: Integer);
    procedure GameNewTurnExecute(Sender: TObject);
    procedure AutoSave;
    procedure CommandLineParams;
    procedure SelectClient;
    procedure UpdateOtherForms;
    procedure FormClientActions;
    procedure LoadGameSystems;
  public
    Game: TGame;
    GameSettings: TGame;
    Server: TServer;
    UseSingleView: Boolean;
    DevelMode: Boolean;
    AnimationSpeed: Integer;
    CellGridVisible: Boolean;
    UnitShapeVisible: Boolean;
    AutoSaveEnabled: Boolean;
    ReopenLastFile: Boolean;
    FormClients: TFormClients;
    LocalClients: TClients;
    ServerList: TServerList;
    GameSystems: TGameSystems;
    FormMain: TFormMain;
    procedure ReopenGameOnInit;
    procedure LoadConfig;
    procedure SaveConfig;
    procedure SaveConfigAll;
    procedure Spectate(Player: TPlayer);
    procedure UpdateInterface;
    procedure ScaleDPI;
    procedure Init;
    procedure Done;
    property Initialized: Boolean read FInitialized;
  end;

var
  Core: TCore;

const
  clOrange = $009Aff;
  clBrown = $a52A2A;


implementation

{$R *.lfm}

uses
  FormNew, FormSettings, ClientAI, FormKeyShortcuts, Common, FormEx, FormTests,
  FormHelp, FormUnitMoves, FormPlayersStats, ClientGUI, FormAbout, Tests,
  FormGameSystems;

const
  DefaultRegKey = '\Software\Chronosoft\xTactics';

resourcestring
  SPlayerWins = 'Player %s wins';
  SPlayersWins = 'Players %s win';
  SEndGame = 'End game?';
  SEndGameQuestion = 'Do you want to end current game?';
  SRestartGame = 'Restart game?';
  SRestartGameQuestion = 'Do you want to restart current game?';
  SPlayersNotInitialized = 'Not all players were initialized with start cell. Needed %d, initialized %d. Change map parameters to have more terrain cells.';
  SFileDialogFilter = 'xTactics games (.xtg)|*.xtg|All files|*.*';
  SMissingServerClientForPlayer = 'Server client for current player not found.';

procedure Translate;
begin
  Game.InitStrings;
end;

{ TCore }

procedure TCore.DoOnWin(Player: TPlayer);
var
  PlayerNames: string;
  AlivePlayers: TPlayerArray;
  I: Integer;
begin
  Server.GameEnd;
  if Assigned(Player) then
    ShowMessage(Format(SPlayerWins, [Player.Name]))
  else begin
    AlivePlayers := Game.Players.GetAlivePlayers;
    PlayerNames := '';
    for I := 0 to Length(AlivePlayers) - 1 do begin
      if I > 0 then PlayerNames := PlayerNames + ', ';
      PlayerNames := PlayerNames + AlivePlayers[I].Name;
    end;
    ShowMessage(Format(SPlayersWins, [PlayerNames]))
  end;
end;

procedure TCore.Delay(Time: Integer);
const
  Slice = 50; // ms
begin
  while Time > 0 do begin
    Application.ProcessMessages;
    if Time > Slice then Sleep(Slice) else Sleep(Time);
    Dec(Time, Slice);
  end;
end;

procedure TCore.GameNewTurnExecute(Sender: TObject);
begin
  if AutoSaveEnabled then AutoSave;
end;

procedure TCore.AutoSave;
var
  OldFileName: string;
begin
  OldFileName := Game.FileName;
  Game.SaveToFile(GetAppConfigDir(False) + 'AutoSave.xtg');
  Game.FileName := OldFileName;
end;

procedure TCore.LoadConfig;
begin
  RegistryContext := TRegistryContext.Create(HKEY_CURRENT_USER, DefaultRegKey);
  PersistentForm1.RegistryContext := RegistryContext;
  XMLConfig1.Filename := GetAppConfigDir(False) + 'Config.xml';

  ThemeManager1.Theme := ThemeManager1.Themes.FindByName(String(XmlConfig1.GetValue('Theme', 'System')));
  LastOpenedList1.LoadFromXMLConfig(XMLConfig1, 'RecentFiles');
  DevelMode := XMLConfig1.GetValue('DevelMode', false);
  AnimationSpeed := XMLConfig1.GetValue('AnimationSpeed', 50);
  AutoSaveEnabled := XMLConfig1.GetValue('AutoSave', True);
  ReopenLastFile := XMLConfig1.GetValue('ReopenLastFile', True);
  Translator1.Language := Translator1.Languages.SearchByCode(String(XMLConfig1.GetValue('Language', '')));
  ScaleDPI1.DPI := Point(XMLConfig1.GetValue('DPIX', 96), XMLConfig1.GetValue('DPIY', 96));
  ScaleDPI1.AutoDetect := XMLConfig1.GetValue('DPIAuto', True);
  FormNewTabIndex := XMLConfig1.GetValue('FormNewTabIndex', 0);
  CellGridVisible := XMLConfig1.GetValue('CellGridVisible', True);
  UnitShapeVisible := XMLConfig1.GetValue('UnitShapeVisible', False);
end;

procedure TCore.SaveConfig;
begin
  XMLConfig1.SetValue('Language', WideString(Translator1.Language.Code));
  XMLConfig1.SetValue('Theme', WideString(ThemeManager1.Theme.Name));
  LastOpenedList1.SaveToXMLConfig(XMLConfig1, 'RecentFiles');
  XMLConfig1.SetValue('DevelMode', DevelMode);
  XMLConfig1.SetValue('AnimationSpeed', AnimationSpeed);
  XMLConfig1.SetValue('AutoSave', AutoSaveEnabled);
  XMLConfig1.SetValue('ReopenLastFile', ReopenLastFile);
  XMLConfig1.SetValue('DPIX', ScaleDPI1.DPI.X);
  XMLConfig1.SetValue('DPIY', ScaleDPI1.DPI.Y);
  XMLConfig1.SetValue('DPIAuto', ScaleDPI1.AutoDetect);
  XMLConfig1.SetValue('FormNewTabIndex', FormNewTabIndex);
  XMLConfig1.SetValue('CellGridVisible', CellGridVisible);
  XMLConfig1.SetValue('UnitShapeVisible', UnitShapeVisible);
end;

procedure TCore.SaveConfigAll;
begin
  GameSettings.SaveConfig(XMLConfig1, 'Game');
  Server.SaveConfig(XMLConfig1, 'Server');
  ServerList.SaveConfig(XmlConfig1, 'ServerList');
  SaveConfig;
  XMLConfig1.Flush;
end;

procedure TCore.CommandLineParams;
var
  FileName: string;
begin
  // Command line parameter handling
  if (ParamCount > 0) then begin
    FileName := ParamStr(1);
    if FileExists(FileName) then LoadGame(FileName);
  end;
end;

procedure TCore.ScaleDPI;
var
  I: Integer;
begin
  //{$ifdef DEBUG}
  with Core.ScaleDPI1 do begin
    //DesignDPI := Point(144, 144);
    if (DesignDPI.X <> DPI.X) or (DesignDPI.Y <> DPI.Y) then begin
      //ApplyToAll(DesignDPI);
      for I := 0 to Screen.FormCount - 1 do
      if (Screen.Forms[I].WindowState = wsNormal) or
        (Screen.Forms[I].WindowState = wsMinimized) then begin
        StoreDimensions(Screen.Forms[I], StoredDimension);
        ScaleDimensions(Screen.Forms[I], StoredDimension);
      end;
      ScaleImageList(Core.ImageListSmall, DesignDPI);
      ScaleImageList(Core.ImageListLarge, DesignDPI);
    end;
  end;
  //{$endif}
end;

procedure TCore.SelectClient;
var
  FirstHuman: TPlayer;
begin
  FirstHuman := Game.Players.GetFirstHuman;
  if Assigned(FirstHuman) then FormMain.FormClient.Client := TClientGUI(LocalClients.SearchPlayer(FirstHuman))
    else begin
      FormMain.FormClient.Client := TClientGUI(LocalClients.New(SSpectator));
      FormMain.FormClient.Client.LocalServer := Server;
      FormMain.FormClient.Client.ConnectType := ctLocal;
      FormMain.FormClient.Client.Active := True;
      FormMain.FormClient.AZoomAll.Execute;
    end;
end;

procedure TCore.UpdateInterface;
begin
  AGameEnd.Enabled := Game.Running;
  AFullscreen.Checked := FormMain.FullScreen;
  {$ifdef DEBUG}
  ATests.Visible := True;
  ATests.Enabled := True;
  {$else}
  ATests.Visible := False;
  ATests.Enabled := False;
  {$endif}
end;

procedure TCore.AExitExecute(Sender: TObject);
begin
  Game.Running := False;
  FormMain.Close;
  Application.Terminate;
end;

procedure TCore.AFullscreenExecute(Sender: TObject);
begin
  FormMain.ToggleFullscreen;
  UpdateInterface;
end;

procedure TCore.AAboutExecute(Sender: TObject);
var
  FormAbout: TFormAbout;
begin
  FormAbout := TFormAbout.Create(nil);
  try
    FormAbout.ApplicationInfo := ApplicationInfo;
    FormAbout.ShowModal;
  finally
    FormAbout.Free;
  end;
end;

procedure TCore.AGameEndExecute(Sender: TObject);
begin
  if MessageDlg(SEndGame, SEndGameQuestion, mtConfirmation, mbYesNo, 0) = mrYes then begin
    Game.Running := False;
    Server.GameEnd;
    UpdateInterface;
  end;
end;

procedure TCore.AGameLoadExecute(Sender: TObject);
begin
  OpenDialog1.Filter := SFileDialogFilter;
  OpenDialog1.InitialDir := ExtractFileDir(Game.FileName);
  OpenDialog1.FileName := ExtractFileName(Game.FileName);
  if OpenDialog1.Execute then begin
    LoadGame(OpenDialog1.FileName);
  end;
end;

procedure TCore.AGameNewExecute(Sender: TObject);
var
  NewServer: TServer;
  FormNew: TFormNew;
begin
  FormNew := TFormNew.Create(nil);
  try
    NewServer := TServer.Create;
    NewServer.Assign(Server);
    NewServer.Game := TGame.Create;
    NewServer.Game.Assign(GameSettings);
    FormNew.Server := NewServer;
    FormNew.PageControl1.TabIndex := FormNewTabIndex;
    if FormNew.ShowModal = mrOk then begin
      Server.Assign(NewServer);
      Server.Game := Game;
      Game.Running := False;
      Game.PostConfig;
      Game.Assign(NewServer.Game);
      GameSettings.Assign(NewServer.Game);
      SaveConfigAll;
      StartNewGame;
      GameLoaded := False;
    end;
    FormNewTabIndex := FormNew.PageControl1.TabIndex;
    NewServer.Game.Free;
    NewServer.Game := nil;
    NewServer.Free;
  finally
    FormNew.Free;
  end;
end;

procedure TCore.AGameRestartExecute(Sender: TObject);
begin
  if MessageDlg(SRestartGame, SRestartGameQuestion, mtConfirmation, mbYesNo, 0) = mrYes then begin
    StartNewGame;
  end;
end;

procedure TCore.AGameSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.Filter := SFileDialogFilter;
  SaveDialog1.InitialDir := ExtractFileDir(Game.FileName);
  SaveDialog1.FileName := ExtractFileName(Game.FileName);
  if SaveDialog1.Execute then begin
    Game.SaveToFile(SaveDialog1.FileName);
    LastOpenedList1.AddItem(SaveDialog1.FileName);
  end;
end;

procedure TCore.AGameSaveExecute(Sender: TObject);
begin
  if FileExists(Game.FileName) then
    Game.SaveToFile(Game.FileName)
    else AGameSaveAs.Execute;
end;

procedure TCore.AGameSystemsExecute(Sender: TObject);
var
  FormGameSystems: TFormGameSystems;
begin
  FormGameSystems := TFormGameSystems.Create(nil);
  try
    FormGameSystems.GameSystems := GameSystems;
    FormGameSystems.ShowModal;
  finally
    FreeAndNil(FormGameSystems);
  end;
end;

procedure TCore.AHelpExecute(Sender: TObject);
var
  FormHelp: TFormHelp;
begin
  FormHelp := TFormHelp.Create(Self);
  try
    FormHelp.ShowModal;
  finally
    FreeAndNil(FormHelp);
  end;
end;

procedure TCore.ANewSpectatorClientExecute(Sender: TObject);
begin
  Spectate(nil);
end;

procedure TCore.APlayersStatsExecute(Sender: TObject);
begin
  if not Assigned(FormMain.FormPlayersStats) then
    FormMain.FormPlayersStats := TFormPlayersStats.Create(nil);
  FormMain.FormPlayersStats.OnSpectate := Spectate;
  FormMain.FormPlayersStats.Game := Game;
  FormMain.FormPlayersStats.Show;
end;

procedure TCore.ASettingsExecute(Sender: TObject);
var
  FormSettings: TFormSettings;
begin
  FormSettings := TFormSettings.Create(nil);
  try
    FormSettings.Load;
    if FormSettings.ShowModal = mrOk then begin
      FormSettings.Save;
      Game.SaveConfig(XMLConfig1, 'Game');
      Server.SaveConfig(XMLConfig1, 'Server');
      XMLConfig1.Flush;
      ThemeManager1.UseTheme(FormMain);
    end;
  finally
    FreeAndNil(FormSettings);
  end;
end;

procedure TCore.AShowChartsExecute(Sender: TObject);
begin
  if not Assigned(FormMain.FormCharts) then
    FormMain.FormCharts := TFormCharts.Create(nil);
  FormMain.FormCharts.Game := Game;
  FormMain.FormCharts.Show;
end;

procedure TCore.AShowKeyShortcutsExecute(Sender: TObject);
begin
  if not Assigned(FormMain.FormKeyShortcuts) then
    FormMain.FormKeyShortcuts := TFormKeyShortcuts.Create(nil);
  with FormMain.FormKeyShortcuts do begin
    SourceComponents.Clear;
    SourceComponents.Add(Self);
    SourceComponents.Add(FormMain);
    Show;
  end;
end;

procedure TCore.AShowUnitMovesExecute(Sender: TObject);
begin
  if not Assigned(FormMain.FormUnitMoves) then
    FormMain.FormUnitMoves := TFormUnitMoves.Create(nil);
  FormMain.FormUnitMoves.Game := Game;
  FormMain.FormUnitMoves.Show;
end;

procedure TCore.ATestsExecute(Sender: TObject);
var
  FormTests: TFormTests;
begin
  FormTests := TFormTests.Create(nil);
  FormTests.TestCases := GetTestCases;
  FormTests.ShowModal;
  FormTests.Free;
end;

procedure TCore.AToggleFogOfWarExecute(Sender: TObject);
begin
  Game.FogOfWar := not Game.FogOfWar;
  Server.DoChange(Self);
end;

procedure TCore.Translator1Translate(Sender: TObject);
begin
  Translate;
  if Assigned(FormMain) then
    Translator1.TranslateComponentRecursive(FormMain.FormClient);
end;

procedure TCore.DataModuleCreate(Sender: TObject);
{$IFDEF Linux}
const
  LinuxGameFilesDir = '/usr/share/xtactics';
  LinuxLanguagesDir = LinuxGameFilesDir + '/Languages';
{$ENDIF}
begin
  GameFilesDir := '';
  {$IFDEF Linux}
  // If installed in Linux system then use installation shared game directory for data files
  if DirectoryExists(LinuxGameFilesDir) then
    GameFilesDir := LinuxGameFilesDir;
  // If installed in Linux system then use installation directory for po files
  if not DirectoryExists(Translator1.POFilesFolder) and DirectoryExists(LinuxLanguagesDir) then
    Translator1.POFilesFolder := LinuxLanguagesDir;
  {$ENDIF}

  GameSystems := TGameSystems.Create;
  Server := TServer.Create;
  LocalClients := TClients.Create;
  GameSettings := TGame.Create;
  GameSettings.FileName := 'Settings';
  Game := TGame.Create;
  Game.OnMoveUpdated := DoMoveUpdated;
  Game.OnWin := DoOnWin;
  Game.OnNewTurn := GameNewTurnExecute;
  Game.OnPlayerChange := DoPlayerChange;
  Game.MapImageFileName := CombinePaths(GameFilesDir, 'Images' + DirectorySeparator + 'Maps' + DirectorySeparator + 'WorldMap.png');
  Server.Game := Game;
  LocalClients.Game := Game;
  StoredDimension := TControlDimension.Create;
  XMLConfig1.Filename := GetAppConfigDir(False) + 'Config.xml';
  ForceDirectories(GetAppConfigDir(False));
  FormClients := TFormClients.Create;
  ServerList := TServerList.Create;

  LoadConfig;

  TFormEx.ScaleDPI := ScaleDPI1;
  TFormEx.Translator := Translator1;
  TFormEx.ThemeManager := ThemeManager1;
  TFormEx.PersistentForm := PersistentForm1;

  Application.CreateForm(TFormMain, FormMain);
  FormMain.Show;
end;

procedure TCore.DataModuleDestroy(Sender: TObject);
begin
  FreeAndNil(ServerList);
  FreeAndNil(FormClients);
  FreeAndNil(StoredDimension);
  FreeAndNil(Server);
  FreeAndNil(LocalClients);
  FreeAndNil(Game);
  FreeAndNil(GameSettings);
  FreeAndNil(GameSystems);
end;

procedure TCore.LastOpenedList1Change(Sender: TObject);
begin
  if Assigned(FormMain) then
    LastOpenedList1.LoadToMenuItem(FormMain.MenuItemLoadRecent, LoadRecentExecute);
end;

procedure TCore.LoadRecentExecute(Sender: TObject);
begin
  LoadGame((Sender as TMenuItem).Caption);
end;

procedure TCore.LoadGame(FileName: string);
var
  I: Integer;
  NewClient: TClient;
  ServerClient: TServerClient;
  Player: TPlayer;
begin
  GameLoaded := True;

  Game.LoadFromFile(FileName);
  Server.InitClients;
  Server.Active := True;

  // Create local LocalClients for human players
  LocalClients.Clear;
  FormMain.FormClient.Client := nil;
  for I := 0 to Game.Players.Count - 1 do begin
    Player := TPlayer(Game.Players[I]);
    with Player do
    if Mode = pmHuman then begin
      NewClient := LocalClients.New(Name);
      NewClient.ControlPlayer := Player;
      TClientGUI(NewClient).View.Clear;
      TClientGUI(NewClient).View.Zoom := 1;
      NewClient.LocalServer := Server;
      NewClient.ConnectType := ctLocal;
      NewClient.Active := True;
      if Assigned(NewClient.ControlPlayer.StartCell) then
        TClientGUI(NewClient).View.CenterPlayerCity(NewClient.ControlPlayer)
        else TClientGUI(NewClient).View.CenterMap;
    end else
    if Mode = pmComputer then begin
      NewClient := TComputer.Create;
      NewClient.Game := TGame(Game);
      NewClient.Name := Name;
      LocalClients.Add(NewClient);
      NewClient.ControlPlayer := Player;
      NewClient.LocalServer := Server;
      NewClient.ConnectType := ctLocal;
      NewClient.Active := True;
    end;
  end;

  SelectClient;
  LastOpenedList1.AddItem(FileName);
  UpdateOtherForms;
  UpdateInterface;
  ServerClient := Server.Clients.SearchByPlayer(Game.CurrentPlayer);
  if Assigned(ServerClient) then ServerClient.TurnStart
    else raise Exception.Create(SMissingServerClientForPlayer);
  FormMain.FormClient.AZoomAll.Execute;
end;

procedure TCore.UpdateOtherForms;
begin
  if Assigned(FormMain.FormCharts) and FormMain.FormCharts.Visible then
    FormMain.FormCharts.Redraw;
  if Assigned(FormMain.FormUnitMoves) and FormMain.FormUnitMoves.Visible then
    FormMain.FormUnitMoves.ReloadList;
  if Assigned(FormMain.FormPlayersStats) and FormMain.FormPlayersStats.Visible then
    FormMain.FormPlayersStats.ReloadList;
end;

procedure TCore.FormClientActions;
var
  Action: TAction;
  I: Integer;
begin
  // Copy all actions from docked form to main form so keyboard shortcuts will be active
  for I := 0 to FormMain.FormClient.ActionList1.ActionCount - 1 do begin
    Action := TAction.Create(FormMain);
    Action.Assign(TAction(FormMain.FormClient.ActionList1.Actions[I]));
    Action.ActionList := FormMain.ActionList1;
  end;
end;

procedure TCore.LoadGameSystems;
var
  NewGameSystem: TGameSystem;
  NewUnitKind: TUnitKind;
  SystemsDir: string;
const
  GameSystemsDirName = 'Systems';
  DefaultGameSystemName = 'Default';
begin
  if GameFilesDir <> '' then
    SystemsDir := GameFilesDir + DirectorySeparator + GameSystemsDirName
    else SystemsDir := GameSystemsDirName;
  if DirectoryExists(SystemsDir) then begin
    GameSystems.LoadFromDir(SystemsDir);
  end else begin
    // Create default game system if the directory with definitions was not found
    NewGameSystem := TGameSystem.Create;
    NewGameSystem.FileName := DefaultGameSystemName;
    NewUnitKind := TUnitKind.Create;
    NewUnitKind.Name := 'Unit';
    NewGameSystem.UnitKinds.Add(NewUnitKind);
    GameSystems.Add(NewGameSystem);
  end;
  if GameSystems.Count > 0 then begin
    GameSettings.GameSystem.Assign(GameSystems.First);
    Game.GameSystem.Assign(GameSystems.First);
  end;
end;

procedure TCore.ReopenGameOnInit;
begin
  CommandLineParams;
  if not GameLoaded and ReopenLastFile and (LastOpenedList1.Items.Count > 0) and
    FileExists(LastOpenedList1.Items[0]) then
    LoadGame(LastOpenedList1.Items[0]);

  if Game.FileName = '' then begin
    GameSettings.PostConfig;
    Game.Assign(GameSettings);
    StartNewGame;
  end;
end;

procedure TCore.Spectate(Player: TPlayer);
var
  Form: TFormClient;
begin
  Form := TFormClient.Create(nil);
  Form.Client := TClientGUI(LocalClients.New(SSpectator));
  Form.Client.LocalServer := Server;
  Form.Client.ConnectType := ctLocal;
  Form.Client.Active := True;
  //Form.Client.Form := Form;
  //Form.Client.ControlPlayer := Player;
  Form.AZoomAll.Execute;
  Form.Show;
  FormClients.Add(Form);
end;

procedure TCore.StartNewGame;
var
  NewClient: TClient;
  Player: TPlayer;
  I: Integer;
  ServerClient: TServerClient;
begin
  Game.New;
  Server.InitClients;
  Server.Active := True;

  // Create local LocalClients for human players
  LocalClients.Clear;
  FormMain.FormClient.Client := nil;
  for I := 0 to Game.Players.Count - 1 do begin
    Player := TPlayer(Game.Players[I]);
    with Player do
    if Mode = pmHuman then begin
      NewClient := LocalClients.New(Name);
      NewClient.ControlPlayer := Player;
      TClientGUI(NewClient).View.Clear;
      TClientGUI(NewClient).View.Zoom := 1;
      NewClient.LocalServer := Server;
      NewClient.ConnectType := ctLocal;
      NewClient.Active := True;
      if Assigned(NewClient.ControlPlayer.StartCell) then
        TClientGUI(NewClient).View.CenterPlayerCity(NewClient.ControlPlayer)
        else TClientGUI(NewClient).View.CenterMap;
    end else
    if Mode = pmComputer then begin
      NewClient := TComputer.Create;
      NewClient.Game := TGame(Game);
      NewClient.Name := Name;
      LocalClients.Add(NewClient);
      NewClient.ControlPlayer := Player;
      NewClient.LocalServer := Server;
      NewClient.ConnectType := ctLocal;
      NewClient.Active := True;
    end;
  end;

  Game.DevelMode := DevelMode;
  SelectClient;
  if Game.Players.GetAliveCount = Game.Players.Count then Game.Running := True
    else ShowMessage(Format(SPlayersNotInitialized, [Game.Players.Count, Game.Players.GetAliveCount]));
  UpdateOtherForms;
  UpdateInterface;
  ServerClient := Server.Clients.SearchByPlayer(Game.CurrentPlayer);
  if Assigned(ServerClient) then ServerClient.TurnStart
    else raise Exception.Create(SMissingServerClientForPlayer);
end;

procedure TCore.DoPlayerChange(Sender: TObject);
var
  PlayerClient: TClient;
  ServerClient: TServerClient;
begin
  Application.ProcessMessages;
  Server.NextPlayer;
  if Assigned(Game.CurrentPlayer) then begin
    if Game.CurrentPlayer.Mode = pmHuman then begin
      PlayerClient := LocalClients.SearchPlayer(Game.CurrentPlayer);
      if Assigned(PlayerClient) then FormMain.FormClient.Client := TClientGUI(PlayerClient);
    end;

    ServerClient := Server.Clients.SearchByPlayer(Game.CurrentPlayer);
    if Assigned(ServerClient) then ServerClient.TurnStart;
  end;
  UpdateOtherForms;
end;

procedure TCore.DoMoveUpdated(UnitMove: TUnitMove);
begin
  if Assigned(FormMain.FormUnitMoves) and FormMain.FormUnitMoves.Visible then
    FormMain.FormUnitMoves.ReloadList;
end;

procedure TCore.Init;
begin
  if not Core.Initialized then begin
    FInitialized := True;
    LoadGameSystems;

    // Update translated default player names
    GameSettings.Players[0].Name := SPlayer + ' 1';
    GameSettings.Players[1].Name := SPlayer + ' 2';

    // GameSystemName needs to be loaded before game systems config itself.
    // Players reference Nations from game system.
    GameSettings.GameSystem.Assign(GameSystems.SearchByName(string(XMLConfig1.GetValue(DOMString('Game/GameSystemName'), DOMString('HexWars')))));
    GameSettings.LoadConfig(XMLConfig1, 'Game');
    Server.LoadConfig(XMLConfig1, 'Server');
    ServerList.LoadConfig(XmlConfig1, 'ServerList');

    FormClientActions;
  end;
end;

procedure TCore.Done;
begin
  SaveConfigAll;
end;

end.

